@@ -20,6 +20,7 @@ import mock
 import six
 
 from novaclient import api_versions
+from novaclient import base
 from novaclient import exceptions
 from novaclient.tests.unit.fixture_data import client
 from novaclient.tests.unit.fixture_data import floatingips
@@ -29,6 +30,21 @@ from novaclient.tests.unit.v2 import fakes
 from novaclient.v2 import servers
 
 
+class _FloatingIPManager(base.Manager):
+    resource_class = base.Resource
+
+    @api_versions.deprecated_after('2.35')
+    def list(self):
+        """DEPRECATED: List floating IPs"""
+        return self._list("/os-floating-ips", "floating_ips")
+
+    @api_versions.deprecated_after('2.35')
+    def get(self, floating_ip):
+        """DEPRECATED: Retrieve a floating IP"""
+        return self._get("/os-floating-ips/%s" % base.getid(floating_ip),
+                         "floating_ip")
+
+
 class ServersTest(utils.FixturedTestCase):
 
     client_fixture_class = client.V1
@@ -40,6 +56,7 @@ class ServersTest(utils.FixturedTestCase):
         self.useFixture(floatingips.FloatingFixture(self.requests_mock))
         if self.api_version:
             self.cs.api_version = api_versions.APIVersion(self.api_version)
+        self.floating_ips = _FloatingIPManager(self.cs)
 
     def _get_server_create_default_nics(self):
         """Callback for default nics kwarg when creating a server.
@@ -539,33 +556,39 @@ class ServersTest(utils.FixturedTestCase):
         self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
 
-    def test_add_fixed_ip(self):
+    @mock.patch('warnings.warn')
+    def test_add_fixed_ip(self, mock_warn):
         s = self.cs.servers.get(1234)
         fip = s.add_fixed_ip(1)
+        mock_warn.assert_called_once()
         self.assert_request_id(fip, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
         fip = self.cs.servers.add_fixed_ip(s, 1)
         self.assert_request_id(fip, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
 
-    def test_remove_fixed_ip(self):
+    @mock.patch('warnings.warn')
+    def test_remove_fixed_ip(self, mock_warn):
         s = self.cs.servers.get(1234)
         ret = s.remove_fixed_ip('10.0.0.1')
+        mock_warn.assert_called_once()
         self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
         ret = self.cs.servers.remove_fixed_ip(s, '10.0.0.1')
         self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
 
-    def test_add_floating_ip(self):
+    @mock.patch('warnings.warn')
+    def test_add_floating_ip(self, mock_warn):
         s = self.cs.servers.get(1234)
         fip = s.add_floating_ip('11.0.0.1')
+        mock_warn.assert_called_once()
         self.assert_request_id(fip, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
         fip = self.cs.servers.add_floating_ip(s, '11.0.0.1')
         self.assert_request_id(fip, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
-        f = self.cs.floating_ips.list()[0]
+        f = self.floating_ips.list()[0]
         fip = self.cs.servers.add_floating_ip(s, f)
         self.assert_request_id(fip, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
@@ -582,7 +605,7 @@ class ServersTest(utils.FixturedTestCase):
                                               fixed_address='12.0.0.1')
         self.assert_request_id(fip, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
-        f = self.cs.floating_ips.list()[0]
+        f = self.floating_ips.list()[0]
         fip = self.cs.servers.add_floating_ip(s, f)
         self.assert_request_id(fip, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
@@ -590,15 +613,17 @@ class ServersTest(utils.FixturedTestCase):
         self.assert_request_id(fip, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
 
-    def test_remove_floating_ip(self):
+    @mock.patch('warnings.warn')
+    def test_remove_floating_ip(self, mock_warn):
         s = self.cs.servers.get(1234)
         ret = s.remove_floating_ip('11.0.0.1')
+        mock_warn.assert_called_once()
         self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
         ret = self.cs.servers.remove_floating_ip(s, '11.0.0.1')
         self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
-        f = self.cs.floating_ips.list()[0]
+        f = self.floating_ips.list()[0]
         ret = self.cs.servers.remove_floating_ip(s, f)
         self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
@@ -794,44 +819,59 @@ class ServersTest(utils.FixturedTestCase):
 
     def test_get_vnc_console(self):
         s = self.cs.servers.get(1234)
-        vc = s.get_vnc_console('fake')
+        vc = s.get_vnc_console('novnc')
         self.assert_request_id(vc, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
 
-        vc = self.cs.servers.get_vnc_console(s, 'fake')
+        vc = self.cs.servers.get_vnc_console(s, 'novnc')
         self.assert_request_id(vc, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
 
     def test_get_spice_console(self):
         s = self.cs.servers.get(1234)
-        sc = s.get_spice_console('fake')
+        sc = s.get_spice_console('spice-html5')
         self.assert_request_id(sc, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
 
-        sc = self.cs.servers.get_spice_console(s, 'fake')
+        sc = self.cs.servers.get_spice_console(s, 'spice-html5')
         self.assert_request_id(sc, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
 
     def test_get_serial_console(self):
         s = self.cs.servers.get(1234)
-        sc = s.get_serial_console('fake')
+        sc = s.get_serial_console('serial')
         self.assert_request_id(sc, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
 
-        sc = self.cs.servers.get_serial_console(s, 'fake')
+        sc = self.cs.servers.get_serial_console(s, 'serial')
         self.assert_request_id(sc, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
 
     def test_get_rdp_console(self):
         s = self.cs.servers.get(1234)
-        rc = s.get_rdp_console('fake')
+        rc = s.get_rdp_console('rdp-html5')
         self.assert_request_id(rc, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
 
-        rc = self.cs.servers.get_rdp_console(s, 'fake')
+        rc = self.cs.servers.get_rdp_console(s, 'rdp-html5')
         self.assert_request_id(rc, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
 
+    def test_get_console_url(self):
+        s = self.cs.servers.get(1234)
+        rc = s.get_console_url('novnc')
+        self.assert_request_id(rc, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/action')
+
+        rc = self.cs.servers.get_console_url(s, 'novnc')
+        self.assert_request_id(rc, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/action')
+
+        # test the case with invalid console type
+        self.assertRaises(exceptions.UnsupportedConsoleType,
+                          s.get_console_url,
+                          'invalid')
+
     def test_create_image(self):
         s = self.cs.servers.get(1234)
         im = s.create_image('123')
@@ -1001,44 +1041,83 @@ class ServersV26Test(ServersTest):
 
     def test_get_vnc_console(self):
         s = self.cs.servers.get(1234)
-        vc = s.get_vnc_console('fake')
+        vc = s.get_vnc_console('novnc')
         self.assert_request_id(vc, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/remote-consoles')
 
-        vc = self.cs.servers.get_vnc_console(s, 'fake')
+        vc = self.cs.servers.get_vnc_console(s, 'novnc')
         self.assert_request_id(vc, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/remote-consoles')
 
+        # test the case with invalid console type
+        self.assertRaises(exceptions.UnsupportedConsoleType,
+                          s.get_vnc_console,
+                          'invalid')
+
     def test_get_spice_console(self):
         s = self.cs.servers.get(1234)
-        sc = s.get_spice_console('fake')
+        sc = s.get_spice_console('spice-html5')
         self.assert_request_id(sc, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/remote-consoles')
 
-        sc = self.cs.servers.get_spice_console(s, 'fake')
+        sc = self.cs.servers.get_spice_console(s, 'spice-html5')
         self.assert_request_id(sc, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/remote-consoles')
 
+        # test the case with invalid console type
+        self.assertRaises(exceptions.UnsupportedConsoleType,
+                          s.get_spice_console,
+                          'invalid')
+
     def test_get_serial_console(self):
         s = self.cs.servers.get(1234)
-        sc = s.get_serial_console('fake')
+        sc = s.get_serial_console('serial')
         self.assert_request_id(sc, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/remote-consoles')
 
-        sc = self.cs.servers.get_serial_console(s, 'fake')
+        sc = self.cs.servers.get_serial_console(s, 'serial')
         self.assert_request_id(sc, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/remote-consoles')
 
+        # test the case with invalid console type
+        self.assertRaises(exceptions.UnsupportedConsoleType,
+                          s.get_serial_console,
+                          'invalid')
+
     def test_get_rdp_console(self):
         s = self.cs.servers.get(1234)
-        rc = s.get_rdp_console('fake')
+        rc = s.get_rdp_console('rdp-html5')
         self.assert_request_id(rc, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/remote-consoles')
 
-        rc = self.cs.servers.get_rdp_console(s, 'fake')
+        rc = self.cs.servers.get_rdp_console(s, 'rdp-html5')
         self.assert_request_id(rc, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/remote-consoles')
 
+        # test the case with invalid console type
+        self.assertRaises(exceptions.UnsupportedConsoleType,
+                          s.get_rdp_console,
+                          'invalid')
+
+    def test_get_console_url(self):
+        s = self.cs.servers.get(1234)
+        vc = s.get_console_url('novnc')
+        self.assert_request_id(vc, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/remote-consoles')
+
+        vc = self.cs.servers.get_console_url(s, 'novnc')
+        self.assert_request_id(vc, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/remote-consoles')
+
+        # test the case with invalid console type
+        self.assertRaises(exceptions.UnsupportedConsoleType,
+                          s.get_console_url,
+                          'invalid')
+        # console type webmks is supported since api version 2.8
+        self.assertRaises(exceptions.UnsupportedConsoleType,
+                          s.get_console_url,
+                          'webmks')
+
 
 class ServersV28Test(ServersV26Test):
 
@@ -1054,6 +1133,21 @@ class ServersV28Test(ServersV26Test):
         self.assert_request_id(mksc, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/remote-consoles')
 
+    def test_get_console_url(self):
+        s = self.cs.servers.get(1234)
+        mksc = s.get_console_url('novnc')
+        self.assert_request_id(mksc, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/remote-consoles')
+
+        mksc = self.cs.servers.get_console_url(s, 'novnc')
+        self.assert_request_id(mksc, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/remote-consoles')
+
+        # test the case with invalid console type
+        self.assertRaises(exceptions.UnsupportedConsoleType,
+                          s.get_console_url,
+                          'invalid')
+
 
 class ServersV214Test(ServersV28Test):
 
@@ -1305,13 +1399,61 @@ class ServersV2_37Test(ServersV226Test):
         self.assertIsInstance(s, servers.Server)
 
     def test_add_floating_ip(self):
-        # self.cs.floating_ips.list() is not available after 2.35
+        # self.floating_ips.list() is not available after 2.35
         pass
 
     def test_add_floating_ip_to_fixed(self):
-        # self.cs.floating_ips.list() is not available after 2.35
+        # self.floating_ips.list() is not available after 2.35
         pass
 
     def test_remove_floating_ip(self):
-        # self.cs.floating_ips.list() is not available after 2.35
+        # self.floating_ips.list() is not available after 2.35
         pass
+
+
+class ServersCreateImageBackupV2_45Test(utils.FixturedTestCase):
+    """Tests the 2.45 microversion for createImage and createBackup
+    server actions.
+    """
+
+    client_fixture_class = client.V1
+    data_fixture_class = data.V1
+    api_version = '2.45'
+
+    def setUp(self):
+        super(ServersCreateImageBackupV2_45Test, self).setUp()
+        self.cs.api_version = api_versions.APIVersion(self.api_version)
+
+    def test_create_image(self):
+        """Tests the createImage API with the 2.45 microversion which
+        does not return the Location header, it returns a json dict in the
+        response body with an image_id key.
+        """
+        s = self.cs.servers.get(1234)
+        im = s.create_image('123')
+        self.assertEqual('456', im)
+        self.assert_request_id(im, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/action')
+        im = s.create_image('123', {})
+        self.assert_request_id(im, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/action')
+        im = self.cs.servers.create_image(s, '123')
+        self.assert_request_id(im, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/action')
+        im = self.cs.servers.create_image(s, '123', {})
+        self.assert_request_id(im, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/action')
+
+    def test_backup(self):
+        s = self.cs.servers.get(1234)
+        # Test backup on the Server object.
+        sb = s.backup('back1', 'daily', 1)
+        self.assertIn('image_id', sb)
+        self.assertEqual('456', sb['image_id'])
+        self.assert_request_id(sb, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/action')
+        # Test backup on the ServerManager.
+        sb = self.cs.servers.backup(s, 'back1', 'daily', 2)
+        self.assertEqual('456', sb['image_id'])
+        self.assert_request_id(sb, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/action')
