@@ -15,11 +15,9 @@
 
 import logging
 
-from keystoneauth1.exceptions import catalog as key_ex
-
 from novaclient import client
 from novaclient import exceptions
-from novaclient.i18n import _LE, _LW
+from novaclient.i18n import _
 from novaclient.v2 import agents
 from novaclient.v2 import aggregates
 from novaclient.v2 import assisted_volume_snapshots
@@ -28,14 +26,8 @@ from novaclient.v2 import cells
 from novaclient.v2 import certs
 from novaclient.v2 import cloudpipe
 from novaclient.v2 import contrib
-from novaclient.v2 import fixed_ips
 from novaclient.v2 import flavor_access
 from novaclient.v2 import flavors
-from novaclient.v2 import floating_ip_dns
-from novaclient.v2 import floating_ip_pools
-from novaclient.v2 import floating_ips
-from novaclient.v2 import floating_ips_bulk
-from novaclient.v2 import fping
 from novaclient.v2 import hosts
 from novaclient.v2 import hypervisors
 from novaclient.v2 import images
@@ -47,9 +39,6 @@ from novaclient.v2 import migrations
 from novaclient.v2 import networks
 from novaclient.v2 import quota_classes
 from novaclient.v2 import quotas
-from novaclient.v2 import security_group_default_rules
-from novaclient.v2 import security_group_rules
-from novaclient.v2 import security_groups
 from novaclient.v2 import server_external_events
 from novaclient.v2 import server_groups
 from novaclient.v2 import server_migrations
@@ -136,11 +125,11 @@ class Client(object):
         """
         if direct_use:
             raise exceptions.Forbidden(
-                403, _LE("'novaclient.v2.client.Client' is not designed to be "
-                         "initialized directly. It is inner class of "
-                         "novaclient. You should use "
-                         "'novaclient.client.Client' instead. Related lp "
-                         "bug-report: 1493576"))
+                403, _("'novaclient.v2.client.Client' is not designed to be "
+                       "initialized directly. It is inner class of "
+                       "novaclient. You should use "
+                       "'novaclient.client.Client' instead. Related lp "
+                       "bug-report: 1493576"))
 
         # NOTE(cyeoh): In the novaclient context (unlike Nova) the
         # project_id is not the same as the tenant_id. Here project_id
@@ -153,7 +142,6 @@ class Client(object):
         self.user_id = user_id
         self.flavors = flavors.FlavorManager(self)
         self.flavor_access = flavor_access.FlavorAccessManager(self)
-        self.images = images.ImageManager(self)
         self.glance = images.GlanceManager(self)
         self.limits = limits.LimitsManager(self)
         self.servers = servers.ServerManager(self)
@@ -161,24 +149,13 @@ class Client(object):
 
         # extensions
         self.agents = agents.AgentsManager(self)
-        self.dns_domains = floating_ip_dns.FloatingIPDNSDomainManager(self)
-        self.dns_entries = floating_ip_dns.FloatingIPDNSEntryManager(self)
         self.cloudpipe = cloudpipe.CloudpipeManager(self)
         self.certs = certs.CertificateManager(self)
-        self.floating_ips = floating_ips.FloatingIPManager(self)
-        self.floating_ip_pools = floating_ip_pools.FloatingIPPoolManager(self)
-        self.fping = fping.FpingManager(self)
         self.volumes = volumes.VolumeManager(self)
         self.keypairs = keypairs.KeypairManager(self)
-        self.networks = networks.NetworkManager(self)
         self.neutron = networks.NeutronManager(self)
         self.quota_classes = quota_classes.QuotaClassSetManager(self)
         self.quotas = quotas.QuotaSetManager(self)
-        self.security_groups = security_groups.SecurityGroupManager(self)
-        self.security_group_rules = \
-            security_group_rules.SecurityGroupRuleManager(self)
-        self.security_group_default_rules = \
-            security_group_default_rules.SecurityGroupDefaultRuleManager(self)
         self.usage = usage.UsageManager(self)
         self.virtual_interfaces = \
             virtual_interfaces.VirtualInterfaceManager(self)
@@ -187,8 +164,6 @@ class Client(object):
         self.hypervisors = hypervisors.HypervisorManager(self)
         self.hypervisor_stats = hypervisors.HypervisorStatsManager(self)
         self.services = services.ServiceManager(self)
-        self.fixed_ips = fixed_ips.FixedIPsManager(self)
-        self.floating_ips_bulk = floating_ips_bulk.FloatingIPBulkManager(self)
         self.os_cache = os_cache
         self.availability_zones = \
             availability_zones.AvailabilityZoneManager(self)
@@ -197,8 +172,8 @@ class Client(object):
             server_migrations.ServerMigrationsManager(self)
 
         # V2.0 extensions:
-        # NOTE(andreykurilin): baremetal and tenant_networks extensions are
-        #   deprecated now, which is why they are not initialized by default.
+        # NOTE(andreykurilin): tenant_networks extension is
+        #   deprecated now, which is why it is not initialized by default.
         self.assisted_volume_snapshots = \
             assisted_volume_snapshots.AssistedSnapshotManager(self)
         self.cells = cells.CellsManager(self)
@@ -208,7 +183,11 @@ class Client(object):
         self.server_external_events = \
             server_external_events.ServerExternalEventManager(self)
 
-        self.logger = logger or logging.getLogger(__name__)
+        if not logger:
+            logger = logging.getLogger(__name__)
+            if not logger.handlers:
+                logger.addHandler(logging.StreamHandler())
+        self.logger = logger
 
         # Add in any extensions...
         if extensions:
@@ -266,21 +245,21 @@ class Client(object):
 
     @property
     def projectid(self):
-        self.logger.warning(_LW("Property 'projectid' is deprecated since "
-                                "Ocata. Use 'project_name' instead."))
+        self.logger.warning(_("Property 'projectid' is deprecated since "
+                              "Ocata. Use 'project_name' instead."))
         return self.project_name
 
     @property
     def tenant_id(self):
-        self.logger.warning(_LW("Property 'tenant_id' is deprecated since "
-                                "Ocata. Use 'project_id' instead."))
+        self.logger.warning(_("Property 'tenant_id' is deprecated since "
+                              "Ocata. Use 'project_id' instead."))
         return self.project_id
 
     def __enter__(self):
-        self.logger.warning(_LW("NovaClient instance can't be used as a "
-                                "context manager since Ocata (deprecated "
-                                "behaviour) since it is redundant in case of "
-                                "SessionClient."))
+        self.logger.warning(_("NovaClient instance can't be used as a "
+                              "context manager since Ocata (deprecated "
+                              "behaviour) since it is redundant in case of "
+                              "SessionClient."))
         return self
 
     def __exit__(self, t, v, tb):
@@ -289,9 +268,9 @@ class Client(object):
 
     def set_management_url(self, url):
         self.logger.warning(
-            _LW("Method `set_management_url` is deprecated since Ocata. "
-                "Use `endpoint_override` argument instead while initializing "
-                "novaclient's instance."))
+            _("Method `set_management_url` is deprecated since Ocata. "
+              "Use `endpoint_override` argument instead while initializing "
+              "novaclient's instance."))
         self.client.set_management_url(url)
 
     def get_timings(self):
@@ -300,23 +279,6 @@ class Client(object):
     def reset_timings(self):
         self.client.reset_timings()
 
-    def has_neutron(self):
-        """Check the service catalog to figure out if we have neutron.
-
-        This is an intermediary solution for the window of time where
-        we still have nova-network support in the client, but we
-        expect most people have neutron. This ensures that if they
-        have neutron we understand, we talk to it, if they don't, we
-        fail back to nova proxies.
-        """
-        try:
-            endpoint = self.client.get_endpoint(service_type='network')
-            if endpoint:
-                return True
-            return False
-        except key_ex.EndpointNotFound:
-            return False
-
     def authenticate(self):
         """Authenticate against the server.
 
@@ -326,5 +288,5 @@ class Client(object):
         Returns on success; raises :exc:`exceptions.Unauthorized` if the
         credentials are wrong.
         """
-        self.logger.warning(_LW(
+        self.logger.warning(_(
             "Method 'authenticate' is deprecated since Ocata."))
